@@ -84,7 +84,7 @@ class ClientTable(object):
 
     def humanPlayerNames(self):
         """returns a list excluding robot players"""
-        return list(x for x in self.playerNames if not x.startswith('ROBOT'))
+        return list(x for x in self.playerNames if not x.startswith('Robot '))
 
     @staticmethod
     def fromList(client, table):
@@ -109,7 +109,7 @@ class Client(pb.Referenceable):
     clients = []
 
     def __init__(self, username=None, intelligence=AIDefault):
-        """username is something like ROBOT 1 or None for the game server"""
+        """username is something like Robot 1 or None for the game server"""
         self.username = username
         self.game = None
         self.intelligence = intelligence(self)
@@ -418,11 +418,7 @@ class Client(pb.Referenceable):
                 robbableTile = move.exposedMeld.pairs[1] # we want it capitalized for a hidden Kong
         elif move.message == Message.AskForClaims:
             withDiscard = game.lastDiscard.element
-        game.winner = myself
-        try:
-            hand = myself.computeHand(withTile=withDiscard, robbedTile=robbableTile)
-        finally:
-            game.winner = None
+        hand = myself.computeHand(withTile=withDiscard, robbedTile=robbableTile, asWinner=True)
         if hand.won:
             if Debug.robbingKong:
                 if move.message == Message.DeclaredKong:
@@ -436,9 +432,8 @@ class Client(pb.Referenceable):
     def __maySayOriginalCall(self):
         """returns True if Original Call is possible"""
         myself = self.game.myself
-        myHand = myself.computeHand()
         for tileName in set(myself.concealedTileNames):
-            if (myHand - tileName).callingHands():
+            if (myself.hand - tileName).callingHands():
                 if Debug.originalCall:
                     self.game.debug('%s may say Original Call' % myself)
                 return True
