@@ -111,7 +111,7 @@ class Game(object):
         self.shouldSave = shouldSave
         self.setHandSeed()
         self.activePlayer = None
-        self.winner = None
+        self.__winner = None
         self.moves = []
         self.myself = None   # the player using this client instance for talking to the server
         self.gameid = gameid
@@ -157,6 +157,19 @@ class Game(object):
             field.updateGUI()
             self.wall.decorate()
 
+    @apply
+    def winner():
+        """the name of the game server this game is attached to"""
+        def fget(self):
+            # pylint: disable=W0212
+            return self.__winner
+        def fset(self, value):
+            # pylint: disable=W0212
+            if self.__winner != value:
+                self.__winner = value
+                value.invalidateHand()
+        return property(**locals())
+
     def addCsvTag(self, tag, forAllPlayers=False):
         """tag will be written to tag field in csv row"""
         if forAllPlayers or self.belongsToHumanPlayer():
@@ -238,7 +251,7 @@ class Game(object):
 
     def losers(self):
         """the 3 or 4 losers: All players without the winner"""
-        return list([x for x in self.players if x is not self.winner])
+        return list([x for x in self.players if x is not self.__winner])
 
     @staticmethod
     def windOrder(player):
@@ -456,7 +469,7 @@ class Game(object):
         else:
             for player in self.players:
                 player.clearHand()
-            self.winner = None
+            self.__winner = None
             if not self.isScoringGame():
                 self.sortPlayers()
             self.hidePopups()
@@ -486,7 +499,7 @@ class Game(object):
         self.notRotated += 1
         self.roundHandCount += 1
         self.handDiscardCount = 0
-        if self.winner and self.winner.wind == 'E':
+        if self.__winner and self.__winner.wind == 'E':
             self.eastMJCount += 1
 
     def needSave(self):
@@ -505,8 +518,8 @@ class Game(object):
         scoretime = datetime.datetime.now().replace(microsecond=0).isoformat()
         with Transaction():
             for player in self.players:
-                if player.handContent:
-                    manualrules = '||'.join(x.rule.name for x in player.handContent.usedRules)
+                if player.hand:
+                    manualrules = '||'.join(x.rule.name for x in player.hand.usedRules)
                 else:
                     manualrules = m18n('Score computed manually')
                 Query("INSERT INTO SCORE "
@@ -514,15 +527,15 @@ class Game(object):
                     "points,payments, balance,rotated,notrotated) "
                     "VALUES(%d,%d,?,?,%d,'%s',%d,'%s','%s',%d,%d,%d,%d,%d)" % \
                     (self.gameid, self.handctr, player.nameid,
-                        scoretime, int(player == self.winner),
+                        scoretime, int(player == self.__winner),
                         WINDS[self.roundsFinished % 4], player.wind, player.handTotal,
                         player.payment, player.balance, self.rotated, self.notRotated),
-                    list([player.handContent.string, manualrules]))
+                    list([player.hand.string, manualrules]))
                 if Debug.scores:
                     self.debug('%s: roundwind=%s playerwind=%s handTotal=%s balance=%s' % (
                         player, WINDS[self.roundsFinished % 4], player.wind,
                         player.handTotal, player.balance))
-                for usedRule in player.handContent.usedRules:
+                for usedRule in player.hand.usedRules:
                     rule = usedRule.rule
                     if rule.score.limits:
                         tag = rule.function.__class__.__name__
@@ -541,18 +554,18 @@ class Game(object):
                 "won,prevailing,wind,points,payments, balance,rotated,notrotated) "
                 "VALUES(%d,1,%d,?,?,%d,'%s',%d,'%s','%s',%d,%d,%d,%d,%d)" % \
                 (self.gameid, self.handctr, player.nameid,
-                    scoretime, int(player == self.winner),
+                    scoretime, int(player == self.__winner),
                     WINDS[self.roundsFinished % 4], player.wind, 0,
                     amount, player.balance, self.rotated, self.notRotated),
-                list([player.handContent.string, offense.name]))
+                list([player.hand.string, offense.name]))
         if InternalParameters.field:
             InternalParameters.field.updateGUI()
 
     def maybeRotateWinds(self):
         """if needed, rotate winds, exchange seats. If finished, update database"""
-        if not self.winner:
+        if not self.__winner:
             return False
-        result = self.winner.wind != 'E' or self.eastMJCount == 9
+        result = self.__winner.wind != 'E' or self.eastMJCount == 9
         if result:
             self.rotateWinds()
         return result
@@ -674,7 +687,7 @@ class Game(object):
         """pay the scores"""
         # pylint: disable=R0912
         # too many branches
-        winner = self.winner
+        winner = self.__winner
         if winner:
             winner.wonCount += 1
             guilty = winner.usedDangerousFrom
@@ -684,7 +697,7 @@ class Game(object):
                 if Debug.dangerousGame:
                     self.debug('%s: winner %s. %s pays for all' % \
                                 (self.handId(), winner, guilty))
-                guilty.handContent.usedRules.append((payAction, None))
+                guilty.hand.usedRules.append((payAction, None))
                 score = winner.handTotal
                 score = score * 6 if winner.wind == 'E' else score * 4
                 guilty.getsPayment(-score)
@@ -693,8 +706,8 @@ class Game(object):
 
         for player1 in self.players:
             if Debug.explain:
-                self.debug('%s: %s' % (player1, player1.handContent.string))
-                for line in player1.handContent.explain():
+                self.debug('%s: %s' % (player1, player1.hand.string))
+                for line in player1.hand.explain():
                     self.debug('   %s' % (line))
             for player2 in self.players:
                 if id(player1) != id(player2):
@@ -887,7 +900,7 @@ class RemoteGame(PlayingGame):
         if self.myself and player != self.myself and not self.playOpen:
             # we are human and server tells us another player discarded a tile. In our
             # game instance, tiles in handBoards of other players are unknown
-            player.concealedTileNames[0] = tileName
+            player.makeTileKnown(tileName)
             result = 'Xy'
         else:
             result = tileName
@@ -945,7 +958,5 @@ class RemoteGame(PlayingGame):
     def saveHand(self):
         """server told us to save this hand"""
         for player in self.players:
-            player.handContent = player.computeHand()
-            if player == self.winner:
-                assert player.handContent.won
+            assert player.hand.won == (player == self.winner)
         Game.saveHand(self)
