@@ -436,12 +436,13 @@ class ClientDialog(QDialog):
                     txt.append(tileTxt)
             txt = '<br><br>'.join(txt)
             tile.graphics.setToolTip(txt)
+        if self.client.game.activePlayer == self.client.game.myself:
+            InternalParameters.field.handSelectorChanged(self.client.game.myself.handBoard)
 
     def checkTiles(self):
         """does the logical state match the displayed tiles?"""
         for player in self.client.game.players:
             logExposed = list()
-            logConcealed = list()
             physExposed = list()
             physConcealed = list()
             for tile in player.bonusTiles:
@@ -453,9 +454,8 @@ class ClientDialog(QDialog):
                     physConcealed.append(tile.element)
             for meld in player.exposedMelds:
                 logExposed.extend(meld.pairs)
-            logConcealed = player.concealedTileNames
+            logConcealed = sorted(player.concealedTileNames)
             logExposed.sort()
-            logConcealed.sort()
             physExposed.sort()
             physConcealed.sort()
             assert logExposed == physExposed, '%s != %s' % (logExposed, physExposed)
@@ -825,7 +825,7 @@ class HumanClient(Client):
     def readyForGameStart(self, tableid, gameid, wantedGame, playerNames, shouldSave=True):
         """playerNames are in wind order ESWN"""
         self.tableList.hide()
-        if sum(not x[1].startswith('ROBOT') for x in playerNames) == 1:
+        if sum(not x[1].startswith('Robot ') for x in playerNames) == 1:
             # we play against 3 robots and we already told the server to start: no need to ask again
             wantStart = True
         else:
@@ -931,8 +931,8 @@ class HumanClient(Client):
     def remote_abort(self, tableid, message, *args):
         """the server aborted this game"""
         if self.table and self.table.tableid == tableid:
-            # translate ROBOT to Roboter:
-            args = [m18nc('kajongg', x) for x in args]
+            # translate Robot to Roboter:
+            args = self.game.players.translatePlayerNames(args)
             logWarning(m18n(message, *args))
             if self.game:
                 self.game.close()
