@@ -21,7 +21,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 import sys
 from collections import defaultdict
 
-from util import logException, logWarning, m18n, m18nc
+from util import logException, logWarning, m18n, m18nc, m18nE
 from common import WINDS, InternalParameters, elements, IntDict, Debug
 from query import Transaction, Query
 from tile import Tile
@@ -96,9 +96,16 @@ class Players(list):
         """return a list of locally defined players like we need them
         for a scoring game"""
         return list(x[0] for x in Query('select name, id from player where'
-                ' not name like "ROBOT %" and not exists(select 1 from'
+                ' not name like "ROBOT %" and not name like "Robot %"'
+                ' and not exists(select 1 from'
                 ' server where server.lastname=player.name)').records)
 
+    def translatePlayerNames(self, names):
+        """for a list of names, translates those names which are english
+        player names into the local language"""
+        known = set(x.name for x in self)
+        return list(self.byName(x).localName if x in known else x for x in names)
+
 class Player(object):
     """all player related attributes without GUI stuff.
     concealedTileNames: used during the hand for all concealed tiles, ungrouped.
@@ -106,10 +113,11 @@ class Player(object):
     containing the concealed melds as the player presents them."""
     # pylint: disable=R0902
     # pylint we need more than 10 instance attributes
+    # pylint: disable=R0904
+    # pylint we need more than 40 public methods
 
-    def __init__(self, game, handContent=None):
+    def __init__(self, game):
         self.game = game
-        self.handContent = handContent
         self.__balance = 0
         self.__payment = 0
         self.wonCount = 0
@@ -128,18 +136,18 @@ class Player(object):
 
     def clearHand(self):
         """clear player attributes concerning the current hand"""
-        self.concealedTileNames = []
-        self.exposedMelds = []
-        self.concealedMelds = []
-        self.bonusTiles = []
+        self.__concealedTileNames = []
+        self.__exposedMelds = []
+        self.__concealedMelds = []
+        self.__bonusTiles = []
         self.discarded = []
         self.visibleTiles.clear()
-        self.handContent = None
+        self.newHandContent = None
         self.originalCallingHand = None
         self.lastTile = None
         self.lastSource = '1'
         self.lastMeld = Meld()
-        self.mayWin = True
+        self.__mayWin = True
         self.__payment = 0
         self.originalCall = False
         self.dangerousTiles = list()
@@ -147,6 +155,66 @@ class Player(object):
         self.playedDangerous = False
         self.usedDangerousFrom = None
         self.isCalling = False
+        self.__hand = None
+
+    def invalidateHand(self):
+        """some source for the computation of current hand changed"""
+        self.__hand = None
+
+    @apply
+    def hand(): # pylint: disable=E0202
+        """a readonly tuple"""
+        def fget(self):
+            # pylint: disable=W0212
+            if not self.__hand:
+                self.__hand = self.computeHand()
+            return self.__hand
+        return property(**locals())
+
+    @apply
+    def bonusTiles(): # pylint: disable=E0202
+        """a readonly tuple"""
+        def fget(self):
+            # pylint: disable=W0212
+            return tuple(self.__bonusTiles)
+        return property(**locals())
+
+    @apply
+    def concealedTileNames(): # pylint: disable=E0202
+        """a readonly tuple"""
+        def fget(self):
+            # pylint: disable=W0212
+            return tuple(self.__concealedTileNames)
+        return property(**locals())
+
+    @apply
+    def exposedMelds(): # pylint: disable=E0202
+        """a readonly tuple"""
+        def fget(self):
+            # pylint: disable=W0212
+            return tuple(self.__exposedMelds)
+        return property(**locals())
+
+    @apply
+    def concealedMelds(): # pylint: disable=E0202
+        """a readonly tuple"""
+        def fget(self):
+            # pylint: disable=W0212
+            return tuple(self.__concealedMelds)
+        return property(**locals())
+
+    @apply
+    def mayWin(): # pylint: disable=E0202
+        """a readonly tuple"""
+        def fget(self):
+            # pylint: disable=W0212
+            return self.__mayWin
+        def fset(self, value):
+            # pylint: disable=W0212
+            if self.__mayWin != value:
+                self.__mayWin = value
+                self.__hand = None
+        return property(**locals())
 
     @apply
     def lastSource(): # pylint: disable=E0202
@@ -170,6 +238,13 @@ class Player(object):
             return Players.allIds[self.name]
         return property(**locals())
 
+    @apply
+    def localName():
+        """the localized name of this player"""
+        def fget(self):
+            return m18nc('kajongg, name of robot player, to be translated', self.name)
+        return property(**locals())
+
     def hasManualScore(self): # pylint: disable=R0201
         """virtual: has a manual score been entered for this game?"""
         # pylint does not recognize that this is overridden by
@@ -185,9 +260,7 @@ class Player(object):
                 return spValue.value()
             if not self.game.isScoringGame() and not self.game.winner:
                 return 0
-            if self.handContent:
-                return self.handContent.total()
-            return 0
+            return self.hand.total()
         return property(**locals())
 
     @apply
@@ -240,8 +313,8 @@ class Player(object):
         """got a tile from wall"""
         self.game.activePlayer = self
         tile = self.game.wall.deal([tileName], deadEnd=deadEnd)[0]
-        self.addConcealedTiles(tile)
         self.lastTile = tile.element
+        self.addConcealedTiles(tile)
         if deadEnd:
             self.lastSource = 'e'
         else:
@@ -259,28 +332,31 @@ class Player(object):
             assert isinstance(tile, Tile)
             tileName = tile.element
             if tile.isBonus():
-                self.bonusTiles.append(tile)
+                self.__bonusTiles.append(tile)
             else:
                 assert tileName.istitle()
-                self.concealedTileNames.append(tileName)
+                self.__concealedTileNames.append(tileName)
+        self.__hand = None
         if data:
             self.syncHandBoard(adding=data)
 
     def addMeld(self, meld):
-        """add meld to this hand in a scoring game"""
-        assert self.game.isScoringGame()
-        if len(meld) == 1 and meld[0].isBonus():
-            self.bonusTiles.append(meld[0])
+        """add meld to this hand in a scoring game
+        also used for the Game instance maintained by the server"""
+        if len(meld.tiles) == 1 and meld[0].isBonus():
+            self.__bonusTiles.append(meld[0])
         elif meld.state == CONCEALED and not meld.isKong():
-            self.concealedMelds.append(meld)
+            self.__concealedMelds.append(meld)
         else:
-            self.exposedMelds.append(meld)
+            self.__exposedMelds.append(meld)
+        self.__hand = None
 
     def remove(self, tile=None, meld=None):
         """remove from my melds or tiles"""
         tiles = [tile] if tile else meld.tiles
         if len(tiles) == 1 and tiles[0].isBonus():
-            self.bonusTiles.remove(tiles[0])
+            self.__bonusTiles.remove(tiles[0])
+            self.__hand = None
             self.syncHandBoard()
             return
         if tile:
@@ -288,26 +364,28 @@ class Player(object):
             assert not self.game.isScoringGame()
             tileName = tile.element
             try:
-                self.concealedTileNames.remove(tileName)
+                self.__concealedTileNames.remove(tileName)
             except ValueError:
                 raise Exception('removeTiles(%s): tile not in concealed %s' % \
-                    (tileName, ''.join(self.concealedTileNames)))
+                    (tileName, ''.join(self.__concealedTileNames)))
         else:
             self.removeMeld(meld)
+        self.__hand = None
         self.syncHandBoard()
 
     def removeMeld(self, meld):
         """remove a meld from this hand in a scoring game"""
         assert self.game.isScoringGame()
-        for melds in [self.concealedMelds, self.exposedMelds]:
+        for melds in [self.__concealedMelds, self.__exposedMelds]:
             for idx, myTile in enumerate(melds):
                 if id(myTile) == id(meld):
                     melds.pop(idx)
+        self.__hand = None
 
     def hasConcealedTiles(self, tileNames, within=None):
         """do I have those concealed tiles?"""
         if within is None:
-            within = self.concealedTileNames
+            within = self.__concealedTileNames
         within = within[:]
         for tileName in tileNames:
             if tileName not in within:
@@ -318,23 +396,43 @@ class Player(object):
     def showConcealedTiles(self, tileNames, show=True):
         """show or hide tileNames"""
         if not self.game.playOpen and self != self.game.myself:
-            if not isinstance(tileNames, list):
+            if not isinstance(tileNames, (list, tuple)):
                 tileNames = [tileNames]
-            assert len(tileNames) <= len(self.concealedTileNames), \
-                '%s: showConcealedTiles %s, we have only %s' % (self, tileNames, self.concealedTileNames)
+            assert len(tileNames) <= len(self.__concealedTileNames), \
+                '%s: showConcealedTiles %s, we have only %s' % (self, tileNames, self.__concealedTileNames)
             for tileName in tileNames:
                 src, dst = ('Xy', tileName) if show else (tileName, 'Xy')
-                assert src != dst, (self, src, dst, tileNames, self.concealedTileNames)
-                if not src in self.concealedTileNames:
+                assert src != dst, (self, src, dst, tileNames, self.__concealedTileNames)
+                if not src in self.__concealedTileNames:
                     logException( '%s: showConcealedTiles(%s): %s not in %s.' % \
-                            (self, tileNames, src, self.concealedTileNames))
-                idx = self.concealedTileNames.index(src)
-                self.concealedTileNames[idx] = dst
+                            (self, tileNames, src, self.__concealedTileNames))
+                idx = self.__concealedTileNames.index(src)
+                self.__concealedTileNames[idx] = dst
+            self.__hand = None
             self.syncHandBoard()
 
+    def showConcealedMelds(self, concealedMelds, ignoreDiscard=None):
+        """the server tells how the winner shows and melds his
+        concealed tiles. In case of error, return message and arguments"""
+        for part in concealedMelds.split():
+            meld = Meld(part)
+            for pair in meld.pairs:
+                if pair == ignoreDiscard:
+                    ignoreDiscard = None
+                else:
+                    if not pair in self.__concealedTileNames:
+                        msg = m18nE('%1 claiming MahJongg: She does not really have tile %2')
+                        return msg, self.name, pair
+                    self.__concealedTileNames.remove(pair)
+            self.addMeld(meld)
+        if self.__concealedTileNames:
+            msg = m18nE('%1 claiming MahJongg: She did not pass all concealed tiles to the server')
+            return msg, self.name
+        self.__hand = None
+
     def hasExposedPungOf(self, tileName):
         """do I have an exposed Pung of tileName?"""
-        for meld in self.exposedMelds:
+        for meld in self.__exposedMelds:
             if meld.pairs == [tileName.lower()] * 3:
                 return True
         return False
@@ -343,7 +441,7 @@ class Player(object):
         """used for robbing the kong"""
         assert tileName.istitle()
         tileName = tileName.lower()
-        for meld in self.exposedMelds:
+        for meld in self.__exposedMelds:
             if tileName in meld.pairs:
                 meld.pairs.remove(tileName)
                 meld.meldtype = PUNG
@@ -366,12 +464,11 @@ class Player(object):
         """do we compute the same score as the server does?"""
         if score is None:
             return True
-        if 'Xy' in self.concealedTileNames:
+        if 'Xy' in self.__concealedTileNames:
             return True
-        self.handContent = self.computeHand()
-        if str(self.handContent) == score:
+        if str(self.hand) == score:
             return True
-        self.game.debug('%s localScore:%s' % (self, self.handContent))
+        self.game.debug('%s localScore:%s' % (self, self.hand))
         self.game.debug('%s serverScore:%s' % (self, score))
         logWarning('Game %s: client and server disagree about scoring, see logfile for details' % self.game.seed)
         return False
@@ -384,7 +481,7 @@ class Player(object):
             # declaring a kong is never dangerous because we get
             # an unknown replacement
             return False
-        afterExposed = list(x.lower() for x in self.concealedTileNames)
+        afterExposed = list(x.lower() for x in self.__concealedTileNames)
         if exposing:
             exposing = exposing[:]
             if self.game.lastDiscard:
@@ -410,19 +507,20 @@ class Player(object):
         if len(allMeldTiles) == 4 and allMeldTiles[0].islower():
             tile0 = allMeldTiles[0].lower()
             # we are adding a 4th tile to an exposed pung
-            self.exposedMelds = [meld for meld in self.exposedMelds if meld.pairs != [tile0] * 3]
+            self.__exposedMelds = [meld for meld in self.__exposedMelds if meld.pairs != [tile0] * 3]
             meld = Meld(tile0 * 4)
-            self.concealedTileNames.remove(allMeldTiles[3])
+            self.__concealedTileNames.remove(allMeldTiles[3])
             self.visibleTiles[tile0] += 1
         else:
             allMeldTiles = sorted(allMeldTiles) # needed for Chow
             meld = Meld(allMeldTiles)
             for meldTile in meldTiles:
-                self.concealedTileNames.remove(meldTile)
+                self.__concealedTileNames.remove(meldTile)
             for meldTile in allMeldTiles:
                 self.visibleTiles[meldTile.lower()] += 1
             meld.expose(bool(calledTile))
-        self.exposedMelds.append(meld)
+        self.__exposedMelds.append(meld)
+        self.__hand = None
         game.computeDangerous(self)
         adding = [calledTile] if calledTile else None
         self.syncHandBoard(adding=adding)
@@ -430,9 +528,9 @@ class Player(object):
 
     def findDangerousTiles(self):
         """update the list of dangerous tile"""
-        pName = m18nc('kajongg', self.name)
+        pName = self.localName
         dangerous = list()
-        expMeldCount = len(self.exposedMelds)
+        expMeldCount = len(self.__exposedMelds)
         if expMeldCount >= 3:
             if all(x in elements.greenHandTiles for x in self.visibleTiles):
                 dangerous.append((elements.greenHandTiles,
@@ -486,7 +584,7 @@ class Player(object):
         """virtual: player gets focus on his hand"""
         pass
 
-    def mjString(self):
+    def mjString(self, asWinner=False):
         """compile hand info into a string as needed by the scoring engine"""
         game = self.game
         assert game
@@ -494,7 +592,7 @@ class Player(object):
         wonChar = 'm'
         lastSource = ''
         declaration = ''
-        if self == game.winner:
+        if asWinner or self == game.winner:
             wonChar = 'M'
             lastSource = self.lastSource
             if self.originalCall:
@@ -503,18 +601,31 @@ class Player(object):
             wonChar = 'x'
         return ''.join([wonChar, winds, lastSource, declaration])
 
-    def computeHand(self, withTile=None, robbedTile=None, dummy=None):
+    def sortMeldsByX(self):
+        """sorts the melds by their position on screen"""
+        if self.game.isScoringGame():
+            # in a real game, the player melds do not have tiles
+            self.__concealedMelds = sorted(self.__concealedMelds, key=lambda x: x[0].xoffset)
+            self.__exposedMelds = sorted(self.__exposedMelds, key=lambda x: x[0].xoffset)
+
+    def makeTileKnown(self, tileName):
+        """used when somebody else discards a tile"""
+        assert self.__concealedTileNames[0] == 'Xy'
+        self.__concealedTileNames[0] = tileName
+        self.__hand = None
+
+    def computeHand(self, withTile=None, robbedTile=None, dummy=None, asWinner=False):
         """returns Hand for this player"""
-        assert not (self.concealedMelds and self.concealedTileNames)
+        assert not (self.__concealedMelds and self.__concealedTileNames)
         assert not isinstance(self.lastTile, Tile)
         assert not isinstance(withTile, Tile)
-        melds = ['R' + ''.join(self.concealedTileNames)]
+        melds = ['R' + ''.join(self.__concealedTileNames)]
         if withTile:
             melds[0] += withTile
-        melds.extend(x.joined for x in self.exposedMelds)
-        melds.extend(x.joined for x in self.concealedMelds)
-        melds.extend(''.join(x.element) for x in self.bonusTiles)
-        mjString = self.mjString()
+        melds.extend(x.joined for x in self.__exposedMelds)
+        melds.extend(x.joined for x in self.__concealedMelds)
+        melds.extend(''.join(x.element) for x in self.__bonusTiles)
+        mjString = self.mjString(asWinner)
         melds.append(mjString)
         if mjString.startswith('M'):
             melds.append('L%s%s' % (withTile or self.lastTile, self.lastMeld.joined))
@@ -525,38 +636,57 @@ class Player(object):
             rules = None
         return Hand.cached(self, ' '.join(melds), computedRules=rules, robbedTile=robbedTile)
 
+    def computeNewHand(self):
+        """returns the new hand. Same as current unless we need to discard. In that
+        case, make an educated guess about the discard. For player==game.myself, use
+        the focussed tile."""
+        hand = self.hand
+        if hand and hand.tileNames and self.__concealedTileNames:
+            if hand.lenOffset == 1 and not hand.won:
+                if self == self.game.myself:
+                    removeTile = self.handBoard.focusTile.element
+                elif self.lastTile:
+                    removeTile = self.lastTile
+                else:
+                    removeTile = self.__concealedTileNames[0]
+                assert removeTile[0] not in 'fy', 'hand:%s remove:%s lastTile:%s' % (
+                    hand, removeTile, self.lastTile)
+                hand -= removeTile
+                assert not hand.lenOffset
+        return hand
+
     def possibleChows(self, tileName=None, within=None):
         """returns a unique list of lists with possible claimable chow combinations"""
-        exposedChows = [x for x in self.exposedMelds if x.isChow()]
+        exposedChows = [x for x in self.__exposedMelds if x.isChow()]
         if len(exposedChows) >= self.game.ruleset.maxChows:
             return []
         if tileName is None:
             tileName = self.game.lastDiscard.element
         if within is None:
-            within = self.concealedTileNames
+            within = self.__concealedTileNames
         within = within[:]
         within.append(tileName)
         return hasChows(tileName, within)
 
     def exposedChows(self):
         """returns a list of exposed chows"""
-        return [x for x in self.exposedMelds if x.isChow()]
+        return [x for x in self.__exposedMelds if x.isChow()]
 
     def possibleKongs(self):
         """returns a unique list of lists with possible kong combinations"""
         kongs = []
         if self == self.game.activePlayer:
             # declaring a kong
-            for tileName in set([x for x in self.concealedTileNames if x[0] not in 'fy']):
-                if self.concealedTileNames.count(tileName) == 4:
+            for tileName in set([x for x in self.__concealedTileNames if x[0] not in 'fy']):
+                if self.__concealedTileNames.count(tileName) == 4:
                     kongs.append([tileName] * 4)
-                elif self.concealedTileNames.count(tileName) == 1 and \
-                        tileName.lower() * 3 in list(x.joined for x in self.exposedMelds):
+                elif self.__concealedTileNames.count(tileName) == 1 and \
+                        tileName.lower() * 3 in list(x.joined for x in self.__exposedMelds):
                     kongs.append([tileName.lower()] * 3 + [tileName])
         if self.game.lastDiscard:
             # claiming a kong
             discardName = self.game.lastDiscard.element.capitalize()
-            if self.concealedTileNames.count(discardName) == 3:
+            if self.__concealedTileNames.count(discardName) == 3:
                 kongs.append([discardName] * 4)
         return kongs
 
@@ -576,30 +706,31 @@ class Player(object):
                 self.lastSource = 'd'
             # the last claimed meld is exposed
             assert lastMeld in melds, '%s: concealed=%s melds=%s lastMeld=%s lastTile=%s withDiscard=%s' % (
-                    self.concealedTileNames, concealed,
+                    self.__concealedTileNames, concealed,
                     meldsContent(melds), ''.join(lastMeld.pairs), lastTile, withDiscard)
             melds.remove(lastMeld)
             self.lastTile = self.lastTile.lower()
             lastMeld.pairs.toLower()
-            self.exposedMelds.append(lastMeld)
+            self.__exposedMelds.append(lastMeld)
             for tileName in lastMeld.pairs:
                 self.visibleTiles[tileName] += 1
         else:
             melds = [Meld(x) for x in concealed.split()]
             self.lastTile = lastTile
             self.lastMeld = lastMeld
-        self.concealedMelds = melds
-        self.concealedTileNames = []
+        self.__concealedMelds = melds
+        self.__concealedTileNames = []
+        self.__hand = None
         self.syncHandBoard()
 
     def scoringString(self):
         """helper for HandBoard.__str__"""
-        if self.concealedMelds:
-            parts = [x.joined for x in self.concealedMelds + self.exposedMelds]
+        if self.__concealedMelds:
+            parts = [x.joined for x in self.__concealedMelds + self.__exposedMelds]
         else:
-            parts = [''.join(self.concealedTileNames)]
-            parts.extend([x.joined for x in self.exposedMelds])
-        parts.extend(''.join(x.element) for x in self.bonusTiles)
+            parts = [''.join(self.__concealedTileNames)]
+            parts.extend([x.joined for x in self.__exposedMelds])
+        parts.extend(''.join(x.element) for x in self.__bonusTiles)
         return ' '.join(parts)
 
     def others(self):
