@@ -278,6 +278,8 @@ class SelectPlayers(SelectRuleset):
 
 class VisiblePlayer(Player):
     """this player instance has a visual representation"""
+    # pylint: disable=R0904
+    # too many public methods
     def __init__(self, game, idx):
         assert game
         self.handBoard = None # because Player.init calls clearHand()
@@ -330,16 +332,15 @@ class VisiblePlayer(Player):
         if not self.handBoard:
             # might happen at program exit
             return
-        self.handContent = self.computeHand()
-        currentScore = self.handContent.score
+        currentScore = self.hand.score
         hasManualScore = self.hasManualScore()
         for box in self.manualRuleBoxes:
-            if box.rule in self.handContent.computedRules:
+            if box.rule in self.hand.computedRules:
                 box.setVisible(True)
                 box.setChecked(True)
                 box.setEnabled(False)
             else:
-                applicable = bool(self.handContent.manualRuleMayApply(box.rule))
+                applicable = bool(self.hand.manualRuleMayApply(box.rule))
                 if hasManualScore:
                     # only those rules which do not affect the score can be applied
                     applicable = applicable and box.rule.hasNonValueAction()
@@ -352,10 +353,10 @@ class VisiblePlayer(Player):
                                 or (self.computeHand(singleRule=box.rule).score > currentScore)
                 box.setApplicable(applicable)
 
-    def __mjstring(self, singleRule):
+    def __mjstring(self, singleRule, asWinner):
         """compile hand info into a string as needed by the scoring engine"""
         winds = self.wind.lower() + 'eswn'[self.game.roundsFinished % 4]
-        if self == self.game.winner:
+        if asWinner or self == self.game.winner:
             wonChar = 'M'
         else:
             wonChar = 'm'
@@ -378,26 +379,26 @@ class VisiblePlayer(Player):
                 declaration = options['declaration']
         return ''.join([wonChar, winds, lastSource, declaration])
 
-    def __lastString(self):
+    def __lastString(self, asWinner):
         """compile hand info into a string as needed by the scoring engine"""
-        if self != self.game.winner:
+        if not asWinner and self != self.game.winner:
             return ''
         lastTile = InternalParameters.field.computeLastTile()
         if not lastTile:
             return ''
         return 'L%s%s' % (lastTile, InternalParameters.field.computeLastMeld().joined)
 
-    def computeHand(self, withTile=None, robbedTile=None, singleRule=None):
+    def computeHand(self, withTile=None, robbedTile=None, singleRule=None, asWinner=False):
         """returns a Hand object, using a cache"""
         game = self.game
         if not game.isScoringGame():
             # maybe we need a more extended class hierarchy for Player, VisiblePlayer, ScoringPlayer,
             # PlayingPlayer but not now. Just make sure that ExplainView can always call the
             # same computeHand regardless of the game type
-            return Player.computeHand(self, withTile=withTile, robbedTile=robbedTile)
+            return Player.computeHand(self, withTile=withTile, robbedTile=robbedTile, asWinner=asWinner)
         if not self.handBoard:
             return None
-        string = ' '.join([self.scoringString(), self.__mjstring(singleRule), self.__lastString()])
+        string = ' '.join([self.scoringString(), self.__mjstring(singleRule, asWinner), self.__lastString(asWinner)])
         if game.eastMJCount == 8 and self == game.winner and self.wind == 'E':
             cRules = [game.ruleset.findRule('XEAST9X')]
         else:
@@ -969,6 +970,7 @@ class PlayField(KXmlGuiWindow):
         self.game.saveHand()
         self.game.maybeRotateWinds()
         self.game.prepareHand()
+        self.game.initHand()
 
     def prepareHand(self):
         """redecorate wall"""
@@ -1026,7 +1028,7 @@ class PlayField(KXmlGuiWindow):
         sBar = self.statusBar()
         if self.game:
             for idx, player in enumerate(self.game.players):
-                sbMessage = m18nc('kajongg', player.name) + ': ' + str(player.balance)
+                sbMessage = player.localName + ': ' + str(player.balance)
                 if sBar.hasItem(idx):
                     sBar.changeItem(sbMessage, idx)
                 else:
