@@ -104,7 +104,7 @@ class RuleItem(RuleTreeItem):
 
     def content(self, column):
         """return the content stored in this node"""
-        colNames = [str(x.toString()) for x in self.parent.parent.parent.rawContent]
+        colNames = [unicode(x.toString()) for x in self.parent.parent.parent.rawContent]
         content = self.rawContent
         if column == 0:
             return m18n(content.name)
@@ -170,7 +170,7 @@ class RuleModel(TreeModel):
         result = QVariant()
         if index.isValid():
             item = index.internalPointer()
-            if role == Qt.DisplayRole:
+            if role in (Qt.DisplayRole, Qt.EditRole):
                 if index.column() == 1:
                     if isinstance(item, RuleItem) and item.rawContent.parType is bool:
                         return QVariant('')
@@ -237,7 +237,7 @@ class RuleModel(TreeModel):
         self.insertRows(0, ruleListItems, rulesetIndex)
         for ridx, ruleList in enumerate(ruleset.ruleLists):
             listIndex = self.index(ridx, 0, rulesetIndex)
-            ruleItems = list([RuleItem(x) for x in ruleList])
+            ruleItems = list([RuleItem(x) for x in ruleList if x.name])
             self.insertRows(0, ruleItems, listIndex)
 
 class EditableRuleModel(RuleModel):
@@ -256,6 +256,7 @@ class EditableRuleModel(RuleModel):
                 dirty = True
                 content.name = english(name)
         elif column == 1 and content.parType:
+            oldParameter = content.parameter
             if content.parType is int:
                 if content.parameter != value.toInt()[0]:
                     dirty = True
@@ -270,6 +271,10 @@ class EditableRuleModel(RuleModel):
                 if content.parameter != unicode(value.toString()):
                     dirty = True
                     content.parameter = unicode(value.toString())
+            message = content.validateParameter()
+            if message:
+                content.parameter = oldParameter
+                dirty = False
         else:
             unitName = str(self.rootItem.content(column).toString())
             dirty, message = content.score.change(unitName, value)
@@ -297,6 +302,7 @@ class EditableRuleModel(RuleModel):
                     dirty, message = self.__setRuleData(column, content, value)
                     if message:
                         KMessageBox.sorry(None, message)
+                        return False
                 else:
                     return False
             elif role == Qt.CheckStateRole:
