@@ -179,7 +179,7 @@ class Table(object):
         number of humans before suspending"""
         result = 4
         if self.preparedGame:
-            result -= sum (x.name.startswith('ROBOT') for x in self.preparedGame.players)
+            result -= sum (x.name.startswith('Robot ') for x in self.preparedGame.players)
         return result
 
     def sendChatMessage(self, chatLine):
@@ -230,9 +230,9 @@ class Table(object):
         # the server and all databases save the english name but we
         # want to make sure a translation exists for the client GUI
         robotNames = [
-            m18ncE('kajongg', 'ROBOT 1'),
-            m18ncE('kajongg', 'ROBOT 2'),
-            m18ncE('kajongg', 'ROBOT 3')]
+            m18ncE('kajongg, name of robot player, to be translated', 'Robot 1'),
+            m18ncE('kajongg, name of robot player, to be translated', 'Robot 2'),
+            m18ncE('kajongg, name of robot player, to be translated', 'Robot 3')]
         while len(names) < 4:
             names.append(robotNames[3 - len(names)])
         result = RemoteGame(names, self.ruleset, client=Client(),
@@ -480,7 +480,7 @@ class Table(object):
             player.mayWin = False
             block.tellAll(player, Message.ViolatedOriginalCall)
         if game.ruleset.mustDeclareCallingHand and not player.isCalling:
-            if player.computeHand().callingHands(mustBeAvailable=True):
+            if player.hand.callingHands(mustBeAvailable=True):
                 player.isCalling = True
                 block.tellAll(player, Message.Calling)
         if txt:
@@ -529,11 +529,11 @@ class Table(object):
         for clientPlayer in self.game.players:
             allPlayerTiles = []
             for player in self.game.players:
-                bonusTileNames = list(x.element for x in player.bonusTiles)
+                bonusTileNames = tuple(x.element for x in player.bonusTiles)
                 if player == clientPlayer or self.game.playOpen:
                     playerTiles = player.concealedTileNames
                 else:
-                    playerTiles = ['Xy'] * 13
+                    playerTiles = ('Xy',) * 13
                 allPlayerTiles.append((player.name, playerTiles + bonusTileNames))
             block.tellPlayer(clientPlayer, Message.SetConcealedTiles, source=allPlayerTiles)
         block.callback(self.dealt)
@@ -668,25 +668,13 @@ class Table(object):
             withDiscard = lastMove.source[0].capitalize()
             lastMove.player.robTile(withDiscard)
         lastMeld = Meld(lastMeld)
-        ignoreDiscard = withDiscard
-        for part in concealedMelds.split():
-            meld = Meld(part)
-            for pair in meld.pairs:
-                if pair == ignoreDiscard:
-                    ignoreDiscard = None
-                else:
-                    if not pair in player.concealedTileNames:
-                        msg = m18nE('%1 claiming MahJongg: She does not really have tile %2')
-                        self.abort(msg, player.name, pair)
-                    player.concealedTileNames.remove(pair)
-            player.concealedMelds.append(meld)
-        if player.concealedTileNames:
-            msg = m18nE('%1 claiming MahJongg: She did not pass all concealed tiles to the server')
-            self.abort(msg, player.name)
+        msgArgs = player.showConcealedMelds(concealedMelds, withDiscard)
+        if msgArgs:
+            self.abort(*msgArgs) # pylint: disable=W0142
         player.declaredMahJongg(concealedMelds, withDiscard, player.lastTile, lastMeld)
-        if not player.computeHand().won:
+        if not player.hand.won:
             msg = m18nE('%1 claiming MahJongg: This is not a winning hand: %2')
-            self.abort(msg, player.name, player.computeHand().string)
+            self.abort(msg, player.name, player.hand.string)
             return
         block = DeferredBlock(self)
         if robbedTheKong:
@@ -792,7 +780,7 @@ class MJServer(object):
 
     def callRemote(self, user, *args, **kwargs):
         """if we still have a connection, call remote, otherwise clean up"""
-        legalTypes = (int, long, basestring, float, list, type(None))
+        legalTypes = (int, long, basestring, float, list, tuple, type(None))
         for arg in args:
             if not isinstance(arg, legalTypes):
                 raise Exception('callRemote got illegal arg: %s %s' % (arg, type(arg)))
